-- for 'Automatic', parse response to /servertime
-- ex. The current time on the server is 06/23 02:15 AM.
-- then compare that to GetDate() -- if day is off, then add/subtract 24 hours then add difference in minutes/60 rounded to nearest hour  that way we avoid the issue of being off a few minutes

local importPath=string.gsub(getfenv(1)._.Name,"%.TimeOffset","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"
-- requires fontMetric
if FontMetric==nil then
	import (importPath.."FontSupport")
	fontMetric=FontMetric()
end
-- requires Spinner
if Spinner==nil then
	import (importPath.."Spinner")
end

-- a small window for setting the TimeZoneOffset
-- has a button to get servertime from chat
TimeZoneWindow=Turbine.UI.Lotro.Window()
TimeZoneWindow:SetText(Resource[language][39])
TimeZoneWindow.TimeZoneCaption=Turbine.UI.Label()
TimeZoneWindow.TimeZoneCaption:SetParent(TimeZoneWindow)
TimeZoneWindow.TimeZoneCaption:SetPosition(10,45)
TimeZoneWindow.TimeZoneCaption:SetMultiline(false)
TimeZoneWindow.TimeZoneCaption:SetFont(Settings.Font)
TimeZoneWindow.TimeZone=Spinner()
TimeZoneWindow.TimeZone:SetParent(TimeZoneWindow)
TimeZoneWindow.Automatic=ScalableButton()
TimeZoneWindow.Automatic:SetParent(TimeZoneWindow)
TimeZoneWindow.Automatic:SetFont(Settings.Font)
TimeZoneWindow.Automatic:SetAutoSize(false)
TimeZoneWindow.TimeOffset=Turbine.UI.Label()
TimeZoneWindow.TimeOffset:SetParent(TimeZoneWindow)
TimeZoneWindow.TimeOffset:SetMultiline(false)
TimeZoneWindow.TimeOffset:SetLeft(TimeZoneWindow.TimeZoneCaption:GetLeft())
TimeZoneWindow.TimeOffset:SetFont(Settings.Font)
TimeZoneWindow.TimeOffset:SetText(Resource[language][42]..": "..tostring(Settings.TimeZoneOffset)) -- servertime is gmt-4

TimeZoneWindow.TimeZone:ClearValues()
for k=14,-12,-1 do -- Baker Islands (US) are -12, Line Islands (Kiribati) are +14, kinda weird
	if k<0 then
		TimeZoneWindow.TimeZone:AddValue("GMT +"..tostring(0-k))
	elseif k==0 then
		TimeZoneWindow.TimeZone:AddValue("GMT")
	else
		TimeZoneWindow.TimeZone:AddValue("GMT -"..tostring(k))
	end
end

TimeZoneWindow.SetFont=function()
	local size=Turbine.UI.Lotro.FontInfo[Settings.Font].size
	fontMetric:SetFont(Settings.Font)
	TimeZoneWindow.TimeZoneCaption:SetFont(Settings.Font)
	TimeZoneWindow.TimeZoneCaption:SetSize(fontMetric:GetTextWidth(Resource[language][39]..": "),size+2)
	TimeZoneWindow.TimeZoneCaption:SetText(Resource[language][39]..":")
	TimeZoneWindow.TimeZone:SetFont(Settings.Font)
	TimeZoneWindow.TimeZone:SetPosition(TimeZoneWindow.TimeZoneCaption:GetLeft()+TimeZoneWindow.TimeZoneCaption:GetWidth(),TimeZoneWindow.TimeZoneCaption:GetTop())
	TimeZoneWindow.Automatic:SetFont(Settings.Font)
	TimeZoneWindow.Automatic:SetText(Resource[language][41])
	TimeZoneWindow.Automatic:SetWidth(fontMetric:GetTextWidth(Resource[language][41])+20)
	TimeZoneWindow.Automatic:SetTop(TimeZoneWindow.TimeZone:GetTop()+TimeZoneWindow.TimeZone:GetHeight()+5)
	TimeZoneWindow.TimeOffset:SetPosition(TimeZoneWindow.TimeZoneCaption:GetLeft(),TimeZoneWindow.Automatic:GetTop()+TimeZoneWindow.Automatic:GetHeight()+10)
	TimeZoneWindow.TimeOffset:SetSize(TimeZoneWindow.TimeZone:GetLeft()+TimeZoneWindow.TimeZone:GetWidth(),size+2)
	TimeZoneWindow:SetSize(TimeZoneWindow.TimeZone:GetLeft()+TimeZoneWindow.TimeZone:GetWidth()+TimeZoneWindow.TimeZoneCaption:GetLeft()*2,TimeZoneWindow.TimeOffset:GetTop()+TimeZoneWindow.TimeOffset:GetHeight()+45)
	TimeZoneWindow.Automatic:SetLeft((TimeZoneWindow:GetWidth()-TimeZoneWindow.Automatic:GetWidth())/2)
	TimeZoneWindow.TimeOffset:SetText(Resource[language][42]..": "..tostring(Settings.TimeZoneOffset))
end

TimeZoneWindow.SetFont()
TimeZoneWindow.TimeZone:SetSelectedIndex(15-Settings.TimeZoneOffset-4) -- server is GMT-4 which is index 17
TimeZoneWindow.TimeZone.SelectedIndexChanged=function()
	-- index 11=GMT+4 (servertime)
	Settings.TimeZoneOffset=11-TimeZoneWindow.TimeZone:GetSelectedIndex()
	if maliceWindow~=nil then
		maliceWindow.displayedHour=nil -- will force attempt to update malice variant if window is displayed, harmless otherwise
	end
	TimeZoneWindow.TimeOffset:SetText(Resource[language][42]..": "..tostring(Settings.TimeZoneOffset))
end
TimeZoneWindow:SetPosition((displayWidth-TimeZoneWindow:GetWidth())/2,(displayHeight-TimeZoneWindow:GetHeight())/2)
TimeZoneWindow:SetZOrder(2) -- put it in front of mission or malice windows

if language==2 then
	TimeZoneWindow.Automatic.QSData="/heureserveur"
elseif language==3 then
	TimeZoneWindow.Automatic.QSData="/serverzeit"
else
	TimeZoneWindow.Automatic.QSData="/servertime"
end
TimeZoneWindow.Automatic.QSType=Turbine.UI.Lotro.ShortcutType.Alias
do
	local sc=Turbine.UI.Lotro.Shortcut(TimeZoneWindow.Automatic.QSType,TimeZoneWindow.Automatic.QSData)
	sc:SetData(TimeZoneWindow.Automatic.QSData) -- fixes a bug with utf-8 special chars in aliases
	TimeZoneWindow.Automatic.QS:SetShortcut(sc)
end
TimeZoneWindow.Automatic.QSAllowDrop=false
TimeZoneWindow.Automatic.QS:SetVisible(true)
TimeZoneWindow.Automatic:SetWantsUpdates(true)

TimeZoneWindow.ChatListener=function(sender,args)
	if (args.ChatType==Turbine.ChatType.Standard or (debug and args.ChatType==Turbine.ChatType.Tell)) then
		--*** need to capture and parse the chat message:
--*** need to check 24 hour time formats - will need a windows client with 24 hour format? not sure if the time/string is determined on the server or the client
		local month,day,hour,minute,ampm
--region, lx, ly, ox, oy, oz = string.match(args.Message,patternString2..trailingString);

		if language==2 then
			--FR=heureserveur= "L'heure actuelle de votre serveur est 26/07, 06:51 PM."
			if string.find(args.Message,"L'heure actuelle de votre serveur est")~=nil then
				day,month,hour,minute,ampm=string.match(args.Message,"L'heure actuelle de votre serveur est (%d+)%/(%d+), (%d+):(%d+) (%a%a).")
			end
		elseif language==3 then
			--DE=serverzeit= "Der Server zeigt die folgende Zeit an: 26. 07., 06: 55 PM."
			if string.find(args.Message,"Der Server zeigt die folgende Zeit an")~=nil then
				day,month,hour,minute,ampm=string.match(args.Message,"Der Server zeigt die folgende Zeit an: (%d+)%. (%d+)%., (%d+): (%d+) (%a%a).")
			end
		else
			--EN=servertime= "The current time on the server is 07/26 04:00 PM."
			if string.find(args.Message,"The current time on the server is")~=nil then
				month,day,hour,minute,ampm=string.match(args.Message,"The current time on the server is (%d+)%/(%d+) (%d+):(%d+) (%a%a).")
			end
		end
		if month~=nil and day~=nil and hour~=nil and minute~=nil and ampm~=nil then
--			Turbine.Shell.WriteLine("month:"..tostring(month).." day:"..tostring(day).." hour:"..tostring(hour).." minute:"..tostring(minute).." ampm:"..tostring(ampm))
			month=tonumber(month)
			day=tonumber(day)
			hour=tonumber(hour)
			minute=tonumber(minute)

			local curDate=Turbine.Engine:GetDate()
			local serverDateTime={}
			serverDateTime.Year=curDate.Year
			if month==12 and day==31 then
				if curDate.Month==1 and curDate.Day==1 then serverDateTime.Year=serverDateTime.Year-1 end
			elseif month==1 and day==1 then
				if curDate.Month==12 and curDate.Day==31 then serverDateTime.Year=serverDateTime.Year+1 end
			end
			serverDateTime.Month=month
			serverDateTime.Day=day
			serverDateTime.Hour=hour
			serverDateTime.Minute=minute
			serverDateTime.Second=0 -- not generated and not needed for this, we only need to be close enough to round to closest hour
			ampm=string.lower(ampm)
			if ampm=="am" then
				if hour==12 then
					serverDateTime.Hour=0
				end
			else
				serverDateTime.Hour=serverDateTime.Hour+12
			end
			local serverEpochTime=getEpochTime(serverDateTime,false)
			local tmpTime=Turbine.Engine:GetDate()
			local localEpochTime=getEpochTime(tmpTime,false)
			-- now determine difference
			local hourDiff=math.floor((localEpochTime-serverEpochTime)/3600+.5)

--Turbine.Shell.WriteLine("hourDiff:"..tostring(hourDiff))
-- hourDiff is now time difference relative to servertime
			-- now find the spinner value that matches the hourDiff - this will not work during DST
--Settings.TimeZoneOffset=11-TimeZoneWindow.TimeZone:GetSelectedIndex()
			hourDiff=hourDiff+11
			if hourDiff>24 then hourDiff=hourDiff-24 end
			if hourDiff<1 then hourDiff=hourDiff+24 end
			if hourDiff>=1 and hourDiff<=27 then
				TimeZoneWindow.TimeZone:SetSelectedIndex(hourDiff)
				TimeZoneWindow.TimeZone.SelectedIndexChanged() -- for some reason this is not firing during SetSelectedIndex
			end
		end
	end
end
TimeZoneWindow.VisibleChanged=function()
	if TimeZoneWindow:IsVisible() then
-- register the chat listener
		AddCallback(Turbine.Chat, "Received", TimeZoneWindow.ChatListener)
	else
-- unregister the chat listener
		RemoveCallback(Turbine.Chat, "Received", TimeZoneWindow.ChatListener)
	end
end

-- +12== 25
-- 25-17=8
-- so +12=offset 8
